@@ -1,21 +1,14 @@
 """
-Filename: arma.py
-Authors: Doc-Jin Jang, Jerry Choi, Thomas Sargent, John Stachurski
-
 Provides functions for working with and visualizing scalar ARMA processes.
 
+TODO: 1. Fix warnings concerning casting complex variables back to floats
+
 """
 import numpy as np
-from numpy import conj, pi, real
-import matplotlib.pyplot as plt
-from scipy.signal import dimpulse, freqz, dlsim
+from .util import check_random_state
 
-# == Ignore unnecessary warnings concerning casting complex variables back to
-# floats == #
-import warnings
-warnings.filterwarnings('ignore')
 
-class ARMA(object):
+class ARMA:
     r"""
     This class represents scalar ARMA(p, q) processes.
 
@@ -26,9 +19,9 @@ class ARMA(object):
 
         X_t = \phi X_{t-1} + \epsilon_t + \theta \epsilon_{t-1}
 
-    where {epsilon_t} is a white noise process with standard deviation
-    sigma.  If phi and theta are arrays or sequences, then the
-    interpretation is the ARMA(p, q) model
+    where :math:`\epsilon_t` is a white noise process with standard
+    deviation :math:`\sigma`.  If phi and theta are arrays or sequences,
+    then the interpretation is the ARMA(p, q) model
 
     .. math::
 
@@ -47,7 +40,7 @@ class ARMA(object):
     Parameters
     ----------
     phi : scalar or iterable or array_like(float)
-        Autocorrelation values for the the autocorrelated variable.
+        Autocorrelation values for the autocorrelated variable.
         See above for explanation.
     theta : scalar or iterable or array_like(float)
         Autocorrelation values for the white noise of the model.
@@ -66,12 +59,44 @@ class ARMA(object):
         processing we desire.  Corresponds with the theta values
 
     """
-
-    def __init__(self, phi, theta=0, sigma=1) :
+    def __init__(self, phi, theta=0, sigma=1):
         self._phi, self._theta = phi, theta
         self.sigma = sigma
         self.set_params()
-    
+
+    def __repr__(self):
+        m = "ARMA(phi=%s, theta=%s, sigma=%s)"
+        return m % (self.phi, self.theta, self.sigma)
+
+    def __str__(self):
+        m = "An ARMA({p}, {q}) process"
+        p = np.asarray(self.phi).size
+        q = np.asarray(self.theta).size
+        return m.format(p=p, q=q)
+
+    # Special latex print method for working in notebook
+    def _repr_latex_(self):
+        m = r"$X_t = "
+        phi = np.atleast_1d(self.phi)
+        theta = np.atleast_1d(self.theta)
+        rhs = ""
+        for (tm, phi_p) in enumerate(phi):
+            # don't include terms if they are equal to zero
+            if abs(phi_p) > 1e-12:
+                rhs += r"%+g X_{t-%i}" % (phi_p, tm+1)
+
+        if rhs[0] == "+":
+            rhs = rhs[1:]  # remove initial `+` if phi_1 was positive
+
+        rhs += r" + \epsilon_t"
+
+        for (tm, th_q) in enumerate(theta):
+            # don't include terms if they are equal to zero
+            if abs(th_q) > 1e-12:
+                rhs += r"%+g \epsilon_{t-%i}" % (th_q, tm+1)
+
+        return m + rhs + "$"
+
     @property
     def phi(self):
         return self._phi
@@ -84,7 +109,7 @@ class ARMA(object):
     @property
     def theta(self):
         return self._theta
-    
+
     @theta.setter
     def theta(self, new_value):
         self._theta = new_value
@@ -137,6 +162,7 @@ class ARMA(object):
             We take psi[0] as unity.
 
         """
+        from scipy.signal import dimpulse
         sys = self.ma_poly, self.ar_poly, 1
         times, psi = dimpulse(sys, n=impulse_length)
         psi = psi[0].flatten()  # Simplify return value into flat array
@@ -151,22 +177,22 @@ class ARMA(object):
 
         .. math::
 
-            f(w) = \sum_k \gamma(k) exp(-ikw)
+            f(w) = \sum_k \gamma(k) \exp(-ikw)
 
         where gamma is the autocovariance function and the sum is over
         the set of all integers.
 
         Parameters
         ----------
-        two_pi : Boolean, optional)
-            Compute the spectral density function over [0, pi] if
-            two_pi is False and [0, 2 pi] otherwise.  Default value is
+        two_pi : Boolean, optional
+            Compute the spectral density function over :math:`[0, \pi]` if
+            two_pi is False and :math:`[0, 2 \pi]` otherwise.  Default value is
             True
         res : scalar or array_like(int), optional(default=1200)
             If res is a scalar then the spectral density is computed at
             `res` frequencies evenly spaced around the unit circle, but
-            if an array the computes the response at the frequencies
-            given by the array
+            if res is an array then the function computes the response
+            at the frequencies given by the array
 
         Returns
         -------
@@ -177,16 +203,17 @@ class ARMA(object):
             The frequency response
 
         """
+        from scipy.signal import freqz
         w, h = freqz(self.ma_poly, self.ar_poly, worN=res, whole=two_pi)
-        spect = h * conj(h) * self.sigma**2
+        spect = h * np.conj(h) * self.sigma**2
 
         return w, spect
 
-    def autocovariance(self, num_autocov=16) :
+    def autocovariance(self, num_autocov=16):
         """
-        Compute the autocovariance function from the ARMA parameters over the
-        integers range(num_autocov) using the spectral density and the inverse
-        Fourier transform.
+        Compute the autocovariance function from the ARMA parameters
+        over the integers range(num_autocov) using the spectral density
+        and the inverse Fourier transform.
 
         Parameters
         ----------
@@ -200,7 +227,7 @@ class ARMA(object):
         # num_autocov should be <= len(acov) / 2
         return acov[:num_autocov]
 
-    def simulation(self, ts_length=90) :
+    def simulation(self, ts_length=90, random_state=None):
         """
         Compute a simulated sample path assuming Gaussian shocks.
 
@@ -209,82 +236,23 @@ class ARMA(object):
         ts_length : scalar(int), optional(default=90)
             Number of periods to simulate for
 
+        random_state : int or np.random.RandomState, optional
+            Random seed (integer) or np.random.RandomState instance to set
+            the initial state of the random number generator for
+            reproducibility. If None, a randomly initialized RandomState is
+            used.
+
         Returns
         -------
         vals : array_like(float)
             A simulation of the model that corresponds to this class
 
         """
+        from scipy.signal import dlsim
+        random_state = check_random_state(random_state)
+
         sys = self.ma_poly, self.ar_poly, 1
-        u = np.random.randn(ts_length, 1) * self.sigma
+        u = random_state.randn(ts_length, 1) * self.sigma
         vals = dlsim(sys, u)[1]
 
         return vals.flatten()
-
-    def plot_impulse_response(self, ax=None, show=True):
-        if show:
-            fig, ax = plt.subplots()
-        ax.set_title('Impulse response')
-        yi = self.impulse_response()
-        ax.stem(list(range(len(yi))), yi)
-        ax.set_xlim(xmin=(-0.5))
-        ax.set_ylim(min(yi)-0.1,max(yi)+0.1)
-        ax.set_xlabel('time')
-        ax.set_ylabel('response')
-        if show:
-            plt.show()
-
-    def plot_spectral_density(self, ax=None, show=True):
-        if show:
-            fig, ax = plt.subplots()
-        ax.set_title('Spectral density')
-        w, spect = self.spectral_density(two_pi=False)
-        ax.semilogy(w, spect)
-        ax.set_xlim(0, pi)
-        ax.set_ylim(0, np.max(spect))
-        ax.set_xlabel('frequency')
-        ax.set_ylabel('spectrum')
-        if show:
-            plt.show()
-
-    def plot_autocovariance(self, ax=None, show=True):
-        if show:
-            fig, ax = plt.subplots()
-        ax.set_title('Autocovariance')
-        acov = self.autocovariance()
-        ax.stem(list(range(len(acov))), acov)
-        ax.set_xlim(-0.5, len(acov) - 0.5)
-        ax.set_xlabel('time')
-        ax.set_ylabel('autocovariance')
-        if show:
-            plt.show()
-
-    def plot_simulation(self, ax=None, show=True):
-        if show:
-            fig, ax = plt.subplots()
-        ax.set_title('Sample path')
-        x_out = self.simulation()
-        ax.plot(x_out)
-        ax.set_xlabel('time')
-        ax.set_ylabel('state space')
-        if show:
-            plt.show()
-
-    def quad_plot(self) :
-        """
-        Plots the impulse response, spectral_density, autocovariance,
-        and one realization of the process.
-
-        """
-        num_rows, num_cols = 2, 2
-        fig, axes = plt.subplots(num_rows, num_cols, figsize=(12, 8))
-        plt.subplots_adjust(hspace=0.4)
-        plot_functions = [self.plot_impulse_response,
-                     self.plot_spectral_density,
-                     self.plot_autocovariance,
-                     self.plot_simulation]
-        for plot_func, ax in zip(plot_functions, axes.flatten()):
-            plot_func(ax, show=False)
-        plt.show()
-
-
