@@ -1,8 +1,4 @@
 """
-Filename: ecdf.py
-
-Authors: Thomas Sargent, John Stachurski
-
 Implements the empirical cumulative distribution function given an array
 of observations.
 
@@ -10,7 +6,8 @@ of observations.
 
 import numpy as np
 
-class ECDF(object):
+
+class ECDF:
     """
     One-dimensional empirical distribution function given a vector of
     observations.
@@ -29,6 +26,13 @@ class ECDF(object):
     def __init__(self, observations):
         self.observations = np.asarray(observations)
 
+    def __repr__(self):
+        return self.__str__()
+
+    def __str__(self):
+        m = "Empirical CDF:\n  - number of observations: {n}"
+        return m.format(n=self.observations.size)
+
     def __call__(self, x):
         """
         Evaluates the ecdf at x
@@ -45,4 +49,3 @@ class ECDF(object):
 
         """
         return np.mean(self.observations <= x)
-
