@@ -1,11 +1,7 @@
 """
-tests for quantecon.ecdf
-
-@author : Spencer Lyon
-@date : 2014-07-31
+Tests for ecdf.py
 
 """
-from __future__ import division
 import unittest
 import numpy as np
 from quantecon import ECDF
